#include <stdio.h> 
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "kuka.h"
#include "capture.h"
#include "config.h"

int calibrage_get();
int calibrage_set();

void keyboard(unsigned char key,int x,int y);
void ShowVideoDisplayImage();
int read_config_file();
int write_config_file();

#define FILENAME "capture.cfg"
#define LINE_LEN 128
#define VAR_NAME_LEN 128

float capture_pos_x, capture_pos_y, capture_pos_z;
extern int argc;
extern char **argv;

int calibrage_get()
{
  if( read_config_file() ) return calibrage_set();
  return 0;
}

int calibrage_set()
{
  //static char mode_op[4]={0};

  capture_init(LARGEUR_IMAGE , HAUTEUR_IMAGE);
  kuka_init();

  /*======= Initialize GLUT ====================================*/
  glutInit( &argc , argv );
  glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE ); /* enable double buffering of video board */

  /*======= Initialize window ==================================*/
  glutInitWindowSize( LARGEUR_IMAGE , HAUTEUR_IMAGE );	/* set window size */
  glutCreateWindow( "Calibrage morpion" );	/* creat window with name */

  /*======= Initialize projection ==============================*/
  glOrtho( 0.0 , LARGEUR_IMAGE-1.0 , 0.0 , HAUTEUR_IMAGE-1.0 , -1.0 , 1.0 );
    
  /*======= Initilize buffer ====================================*/
  glClearColor( 0.0 , 0.0 , 0.0 , 0.0 );
  glClear( GL_COLOR_BUFFER_BIT );
  glutSwapBuffers();
  glFlush();

  /*======= Register callback functions =========================*/
  glutDisplayFunc( ShowVideoDisplayImage );
  glutIdleFunc( ShowVideoDisplayImage );	/* global idle callback */
  glutKeyboardFunc(keyboard);

  /*======= Begin event loop ====================================*/
  /*
    printf("Passer en modeop AUT pour faire le calibrage.\n");
    while( 1 ){
    kuka_get_mode_op(mode_op,4);
    if( strcmp(mode_op,"#T1")==0 )
    break;
    }
  */
  glutMainLoop();
  /*
    printf("Passer en modeop #AUT pour poursuivre le jeux.\n");
    while( 1 ){
    kuka_get_mode_op(mode_op,4);
    if( strcmp(mode_op,"#AUT")==0 )
    break;
    }
  */
  
  kuka_end();
  capture_end();

  if( write_config_file() ) return 1;

  return 0;
}

void keyboard(unsigned char key,int x,int y)
{
  switch(key)
    {
    case 27:
      kuka_get_pos_act( &capture_pos_x, &capture_pos_y, &capture_pos_z);
      kuka_end();
      capture_end();
      if( write_config_file() ) exit(1);
      exit(0);
      break;
    default :
      break;
    }
  glutPostRedisplay();
}

void ShowVideoDisplayImage()
{
  static unsigned char tmp[largeur_image*hauteur_image*12/8];
  static unsigned char out[largeur_image*hauteur_image*24/8];

  int i,j;
  int y,u,v,r,g,b;
  unsigned char *yp,*up,*vp;
  unsigned char *outp;

  capture_grab(tmp,largeur_image,hauteur_image);
  yp=&tmp[0];
  up=&tmp[hauteur_image * largeur_image];
  vp=&tmp[hauteur_image * largeur_image * 5/4]; // 5/4=1.25
  outp=out;

  for(j=0;j<hauteur_image;j++)
    for( i=0;i<largeur_image;i++) {
      if( ( (j-y_offset)%hauteur_case == 0 && x_offset<i && i<largeur_image-x_offset) ||
	  ( (i-x_offset)%largeur_case == 0 && y_offset<j && j<hauteur_image-y_offset) ){
	*outp=0; outp++;
	*outp=255; outp++;
	*outp=0; outp++;      
      }
      else{
	y=yp[j*largeur_image+i];
	u=up[(int)j/2 * largeur_image/2 + (int)i/2];
	v=vp[(int)j/2 * largeur_image/2 + (int)i/2];

	r=y - 0.0009267 * (u-128) + 1.4016868 * (v-128);
	g=y - 0.3436954 * (u-128) - 0.7141690 * (v-128);
	b=y + 1.7721604 * (u-128) + 0.0009902 * (v-128);

	r= r>255?255:r;
	r= r<0?0:r;
	g= g>255?255:g;
	g= g<0?0:g;
	b= b>255?255:b;
	b= b<0?0:b;

	*outp=r; outp++;
	*outp=g; outp++;
	*outp=b; outp++;
      }
    }


  glRasterPos2i( 0 , 0 );
  glDrawPixels( largeur_image , hauteur_image , GL_RGB , GL_UNSIGNED_BYTE , out );
  glFlush();
  glutSwapBuffers();

}

int read_config_file(){
  FILE *file;
  char line[LINE_LEN]={0};
  char var_name[VAR_NAME_LEN]={0};
  float var_val=0;

  file = fopen(FILENAME,"r");
  if (file == NULL){
    return 1;
  }

  while( fgets(line,LINE_LEN,file) != NULL ){
    sscanf(line,"%s %f",var_name,&var_val);

    if( strncmp(var_name,"capture_pos_x",VAR_NAME_LEN)==0 )
      capture_pos_x=var_val;
    else if( strncmp(var_name,"capture_pos_y",VAR_NAME_LEN)==0 )
      capture_pos_y=var_val;
    else if( strncmp(var_name,"capture_pos_z",VAR_NAME_LEN)==0 )
      capture_pos_z=var_val;

    memset(line,LINE_LEN,0);
    memset(var_name,LINE_LEN,0);
    var_val=0;
  }

  fclose(file);
  return 0;
}

int write_config_file(){
  FILE *file;

  file = fopen(FILENAME,"w");
  if( file == NULL )
    return 1;

  fprintf(file,"capture_pos_x %f\n",capture_pos_x);
  fprintf(file,"capture_pos_y %f\n",capture_pos_y);
  fprintf(file,"capture_pos_z %f\n",capture_pos_z);

  fclose(file);
  return 0;
}
