/*! Ce jeux est bas sur le plateau suivant :
 *  La base est centre au coin bas gauche du damier par rapport  Kuka
 *
 *
 *              KUKA
 * X <-                 (0,0)
 *      |*| |*| |*| |*| |
 *                       
 *                      Y
 *
 * La base est centre au coin bas gauche du damier par rapport  Kuka.
 * 
 * {X 942.5909, Y 354.7992, Z 900.0000, A -88.30630, B 0.001254, C -0.002854}
 * 
 * Le point de capture est en 
 * {X 63.742120, Y 66.868870, Z 236.6134} (relatif  la base ci-dessu)
 *
 * Le point zero devrait etre atteint avec les coordonnes suivantes :
 * {X 0, Y 0, Z 0, A -90, B 0, C 180, S 6, T 18}
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "kuka_api.h"

#define X_NUM_CASE 3
#define Y_NUM_CASE 3
#define Z_NUM_CASE 1
#define X_DIM_CASE 80
#define Y_DIM_CASE 80
/*en fait la piece fait 50 de haut mais on met l'amplitude de
 *monte/descente
 */
#define Z_DIM_CASE 120

#define LENTE 5
#define NORMALE 10
#define RAPIDE 15

#define TIMEOUT 12

extern float capture_pos_x,capture_pos_y,capture_pos_z;

int erreur=0;
kukaVar_t *kukaError=NULL;

kukaVar_t vitesse={"VITESSE",{KUKA_INT,{15}}};
kukaVar_t vitesseFlag={"VITESSE_FLAG",{KUKA_BOOL,{1}}};
#define VITESSE vitesse.valeur.kukaVal_u.kukaInt

kukaVar_t base={"BASE",{KUKA_FRAME,{0}}};
#define BASE base.valeur.kukaVal_u.kukaFrame
kukaVar_t baseFlag={"BASE_FLAG",{KUKA_BOOL,{1}}};

kukaVar_t cible={"CIBLE",{KUKA_FRAME,{0}}};
#define CIBLE cible.valeur.kukaVal_u.kukaFrame
kukaVar_t cibleFlag={"CIBLE_FLAG",{KUKA_BOOL,{1}}};

kukaVar_t pince={"PINCE",{KUKA_BOOL,{0}}};
#define PINCE pince.valeur.kukaVal_u.kukaBool
kukaVar_t pinceFlag={"PINCE_FLAG",{KUKA_BOOL,{1}}};

kukaVar_t pos_act={"$POS_ACT",{KUKA_FRAME,{0}}};
#define POS_ACT pos_act.valeur.kukaVal_u.kukaFrame

kukaVar_t mode_op={"$MODE_OP",{KUKA_STRING,{0}}};
#define MODE_OP pos_act.valeur.kukaVal_u.kukaString

#define OUVRE(x) x.valeur.kukaVal_u.kukaBool=(kukaBool_t)1; SET(x);
#define FERME(x) x.valeur.kukaVal_u.kukaBool=(kukaBool_t)0; SET(x);

#define SET( x ) x##Flag.valeur.kukaVal_u.kukaBool=(kukaBool_t)1;\
                 erreur = kuka_setVar ( &x ) || kuka_setVar ( &x##Flag );\
                 alarm(TIMEOUT);\
                 while(x##Flag.valeur.kukaVal_u.kukaBool)\
                     kuka_getVar(&x##Flag);\
                 alarm(0);

#define GET( x ) erreur = kuka_getVar ( &x );

#define STOP_SI_ERREUR if( erreur ){\
                 printf("failed\n");\
                 kuka_displayVar( kukaError );\
                 kuka_end(); exit(1);\
               }


void kuka_init();
void kuka_capture();
void kuka_get_pos_act(float *x, float *y, float *z);
void kuka_get_mode_op(char *_mode_op, int _mode_op_len);
void kuka_prend(int x, int y);
void kuka_pose(int x, int y);
void kuka_end();

void kuka_init(){
  erreur=0;

  printf("kuka_init call getKukaError([...]).\n");
  kuka_getError(&kukaError);
  kuka_displayVar( kukaError );

  /* Initialisation connection */
  printf("kuka_init call initialize(\"kuka\")...");
  erreur = kuka_initialize("kuka");
  STOP_SI_ERREUR;
  printf("succeeded\n");

  return;
}

void kuka_init_all(){
  erreur=0;

  printf("kuka_init call getKukaError([...]).\n");
  kuka_getError(&kukaError);
  kuka_displayVar( kukaError );

  /* Initialisation connection */
  printf("kuka_init call initialize(\"kuka\")...");
  erreur = kuka_initialize("kuka");
  STOP_SI_ERREUR;
  printf("succeeded\n");

  /* Initialisation vitesse */
  VITESSE=RAPIDE;
  printf("kuka_init call setVar(&vitesse)...");
  SET(vitesse);
  STOP_SI_ERREUR;
  printf("succeeded\n");

  /* Initialisation base */
  BASE.x= 942.5909;
  BASE.y= 345.7992;
  BASE.z= 900.0000;
  BASE.a=-88.30630;
  BASE.b= 0.001254;
  BASE.c=-0.002854;
  printf("kuka_init call setVar(&base)...");
  SET(base);
  STOP_SI_ERREUR;
  printf("succeeded\n");

  /* Initialisation cible */
  CIBLE.x= 0;
  CIBLE.y= 0;
  CIBLE.z= 0;
  CIBLE.a=-90;
  CIBLE.b= 0;
  CIBLE.c= 180;
  printf("kuka_init call setVar(&cible)...");
  SET(cible);
  STOP_SI_ERREUR;
  printf("succeeded\n");

  return;
}

void kuka_capture()
{
  /*
    CIBLE.x= 63.742120;
    CIBLE.y= 66.868870;
    CIBLE.z= 236.6134;
  */
  CIBLE.x= capture_pos_x;
  CIBLE.y= capture_pos_y;
  CIBLE.z= capture_pos_z;
  printf("kuka_capture call setVar(&cible)...");
  SET(cible);
  STOP_SI_ERREUR;
  printf("succeeded\n");

  return;
}
void kuka_get_pos_act(float *x, float *y, float *z){
  printf("kuka_get_pos_act call getVar(&pos_act)...");
  GET(pos_act);
  STOP_SI_ERREUR;
  printf("succeeded\n");

  *x=POS_ACT.x;
  *y=POS_ACT.y;
  *z=POS_ACT.z;

  return;
}

void kuka_get_mode_op(char *_mode_op,int _mode_op_len){
  printf("kuka_get_mode_op call getVar(&mode_op)...");
  GET(mode_op);
  STOP_SI_ERREUR;
  printf("succeeded\n");
 
  strncpy(_mode_op,MODE_OP,_mode_op_len);

  return;
}

void kuka_prend(int x, int y){
  erreur=0;

  CIBLE.z=0;
  SET(cible);

  CIBLE.x= X_DIM_CASE/2.0 + x*X_DIM_CASE;
  CIBLE.y= Y_DIM_CASE/2.0 + y*Y_DIM_CASE;
  SET(cible);

  OUVRE(pince);

  VITESSE=LENTE;
  SET(vitesse);

  CIBLE.z= -Z_DIM_CASE;
  SET(cible);

  FERME(pince);

  CIBLE.z= 0;
  SET(cible);

  VITESSE=RAPIDE;
  SET(vitesse);

  return;
}

void kuka_pose(int x, int y){
  erreur=0;

  CIBLE.z=0;
  SET(cible);

  CIBLE.x= X_DIM_CASE/2.0 + x*X_DIM_CASE;
  CIBLE.y= Y_DIM_CASE/2.0 + y*Y_DIM_CASE;
  SET(cible);

  VITESSE=LENTE;
  SET(vitesse);

  CIBLE.z= -Z_DIM_CASE;
  SET(cible);

  OUVRE(pince);

  CIBLE.z= 0;
  SET(cible);

  FERME(pince);

  VITESSE=RAPIDE;
  SET(vitesse);

  return;
}

void kuka_end(){
  printf("kuka_end call uninitialize()...");
  erreur = kuka_uninitialize();
  if( erreur ){
    printf("failed\n");
    kuka_displayVar( kukaError );
  }
  printf("succeeded\n");
  return;
}

