#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "capture.h"
#include "config.h"
/*
 */

void morpion_get_etat(unsigned char*etat,int joueur);
void morpion_init();
void morpion_end();

void morpion_query_patch(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image,int x_min,int y_min,int x_max,int y_max);
void morpion_query_image(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image);

void morpion_query_patch_bin(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image,int x_min,int y_min,int x_max,int y_max);
void morpion_query_image_bin(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image);

void morpion_capture_image_rgb24(unsigned char*out, int largeur_image, int hauteur_image);
void morpion_capture_image_grey(unsigned char*out, int largeur_image, int hauteur_image);
void morpion_capture_image_yuv420p(unsigned char*out,int largeur_image, int hauteur_image);

int morpion_get_min_image_grey(unsigned char*image,int largeur_image,int hauteur_image);
void morpion_get_histogram_grey(int *histo,unsigned char*image,int largeur_image,int hauteur_image);
void morpion_egalize_image_grey(unsigned char*image,int largeur_image,int hauteur_image);
void morpion_invert_image_grey(unsigned char*image,int largeur_image,int hauteur_image);
void morpion_thresh_image_grey(unsigned char*image,int largeur_image,int hauteur_image,int low,int hight);
void morpion_lines_off(unsigned char*image,int largeur_image,int hauteur_image);

void morpion_save_image_grey(char*filename,unsigned char*image,int largeur_image,int hauteur_image);


/****************************************************
 * Fonction de haut niveau
 *
*/
void morpion_init(){
  capture_init(largeur_image , hauteur_image);
}

void morpion_end(){
  capture_end();
}

void morpion_get_etat(unsigned char*etat,int joueur)
{
  unsigned char image[largeur_image*hauteur_image];
  int min;

  morpion_capture_image_grey(image,largeur_image,hauteur_image);
  morpion_save_image_grey("out_1.pnm",image,largeur_image,hauteur_image);

  morpion_lines_off(image,largeur_image,hauteur_image);
  morpion_save_image_grey("out_2.pnm",image,largeur_image,hauteur_image);

  min=morpion_get_min_image_grey(image,largeur_image,hauteur_image);
  morpion_thresh_image_grey(image,largeur_image,hauteur_image,min,min+60);
  morpion_save_image_grey("out_3.pnm",image,largeur_image,hauteur_image);

  morpion_invert_image_grey(image,largeur_image,hauteur_image);
  morpion_save_image_grey("out_4.pnm",image,largeur_image,hauteur_image);
 
  morpion_query_image_bin(etat,joueur,image,largeur_image,hauteur_image);

  return;
}

/****************************************************
 * Procdure permettant de rcuprer l'tat du damier
 * ou d'une case en fonction d'une image en niveau de
 * gris. IL FAUT QUE LES PIONS SOIENT FONCE ET LE FOND
 * CLAIR
*/
void morpion_query_image(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image)
{
  int x,y;
  int i,j;

 for(j=0 ,  y=y_offset; j<hauteur_morpion && y<hauteur_image; j++ ,  y+=hauteur_case)
    for(i=0 ,  x=x_offset; i<largeur_morpion && x<largeur_image; i++ ,  x+=largeur_case){
      morpion_query_patch(etat,joueur,image,largeur_image,hauteur_image,x,y,x+largeur_case,y+hauteur_case);
      etat++;
    }
}

void morpion_query_patch(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image,int x_min,int y_min,int x_max,int y_max)
{
  int i, j, somme=0, aire=(x_max-x_min)*(y_max-y_min);

  for(j=y_min;j<y_max;j++)
    for(i=x_min;i<x_max;i++){
      if( j*largeur_image+i < largeur_image * hauteur_image )
	if(SEUIL_COULEUR(image[j*largeur_image+i])) somme++;
    }
  
  somme=somme*100/aire;
  printf("patch %d %d / %d %d     \tsomme=%d\n",x_min,y_min,x_max,y_max,somme) ;

  if(SEUIL_POURCENT(somme)) *etat = joueur*SEUIL_POURCENT(somme);
}

/****************************************************
 * Procdure permettant de rcuprer l'tat du damier
 * ou d'une case en fonction d'une image binaire.
 * Lorsqu'il y a un pion c'est noir et blanc sinon.
*/
void morpion_query_image_bin(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image)
{
  int x,y;
  int i,j;

 for(j=0 ,  y=y_offset; j<hauteur_morpion && y<hauteur_image; j++ ,  y+=hauteur_case)
    for(i=0 ,  x=x_offset; i<largeur_morpion && x<largeur_image; i++ ,  x+=largeur_case){
      morpion_query_patch_bin(etat,joueur,image,largeur_image,hauteur_image,x,y,x+largeur_case,y+hauteur_case);
      etat++;
    }
}

void morpion_query_patch_bin(unsigned char*etat,int joueur,unsigned char*image,int largeur_image,int hauteur_image,int x_min,int y_min,int x_max,int y_max)
{
  int i, j, somme=0, aire=(x_max-x_min)*(y_max-y_min);

  for(j=y_min;j<y_max;j++)
    for(i=x_min;i<x_max;i++){
      if( j*largeur_image+i < largeur_image * hauteur_image )
	if(image[j*largeur_image+i]==0) somme++;
    }
  
  printf("patch %d %d / %d %d\tsomme=%d aire=%d pourcentage=%d\n",x_min,y_min,x_max,y_max,somme,aire,somme*100/aire) ;
  somme=somme*100/aire;  
  if(SEUIL_POURCENT(somme)) *etat = joueur*SEUIL_POURCENT(somme);
}


/****************************************************
 * Procdure permettant de rcuprer une image depuis
 * la camra en :
 * - yuv420p 12 bits
 * - niveau de gris 8 bits 
 * - rgb 24 bits
 */
void morpion_capture_image_yuv420p(unsigned char*out,int largeur_image, int hauteur_image)
{
  capture_grab(out,largeur_image,hauteur_image);
}

void morpion_capture_image_grey(unsigned char*out, int largeur_image, int hauteur_image)
{
  unsigned char tmp[largeur_image*hauteur_image* 12/8];
  morpion_capture_image_yuv420p(tmp,largeur_image,hauteur_image);
  memcpy(out,tmp,largeur_image*hauteur_image);
}

void morpion_capture_image_rgb24(unsigned char*out, int largeur_image, int hauteur_image)
{
  unsigned char tmp[largeur_image*hauteur_image*12/8];
  int i,j;
  int y,u,v,r,g,b;
  unsigned char *yp,*up,*vp;
  unsigned char *outp;

  morpion_capture_image_yuv420p(tmp,largeur_image,hauteur_image);
  yp=&tmp[0];
  up=&tmp[hauteur_image * largeur_image];
  vp=&tmp[hauteur_image * largeur_image * 5/4]; // 5/4=1.25
  outp=out;

  for(j=0;j<hauteur_image;j++)
    for( i=0;i<largeur_image;i++) {
      y=yp[j*largeur_image+i];
      u=up[(int)j/2 * largeur_image/2 + (int)i/2];
      v=vp[(int)j/2 * largeur_image/2 + (int)i/2];

      r=y - 0.0009267 * (u-128) + 1.4016868 * (v-128);
      g=y - 0.3436954 * (u-128) - 0.7141690 * (v-128);
      b=y + 1.7721604 * (u-128) + 0.0009902 * (v-128);

      r= r>255?255:r;
      r= r<0?0:r;
      g= g>255?255:g;
      g= g<0?0:g;
      b= b>255?255:b;
      b= b<0?0:b;

      *outp=r; outp++;
      *outp=g; outp++;
      *outp=b; outp++;
    }

}

/****************************************************
 * Procdure permettant le traitement de l'image
 * ATTENTION spcifiques  une image en niveau de gris. 
 */
int morpion_get_min_image_grey(unsigned char*image,int largeur_image,int hauteur_image)
{
  int min=255,i;

  for(i=0;i<largeur_image*hauteur_image;i++)
    if( image[i]<min ) min=image[i];

  return min;
}

void morpion_get_histogram_grey(int *histo,unsigned char*image,int largeur_image,int hauteur_image)
{
  int i;

  memset(histo,0,256);
  for(i=0;i<largeur_image*hauteur_image;i++)
    histo[image[i]]++;
}

void morpion_egalize_image_grey(unsigned char*image,int largeur_image,int hauteur_image)
{
  int histo[256];
  unsigned char lut[256];
  int limite=largeur_image*hauteur_image/256;
  int accumulateur;
  int i=0,j=0;

  morpion_get_histogram_grey(histo,image,largeur_image,hauteur_image);

  while(j<255 && i<255){
    accumulateur=histo[i] ;
    while( accumulateur<limite  && i<255 ) {
      lut[i]=j;
      i++;
      accumulateur+=histo[i];
    }
    //i++;
    j++;
  }

  for(i=0;i<largeur_image*hauteur_image;i++)
    image[i]=lut[image[i]];

}

void morpion_thresh_image_grey(unsigned char*image,int largeur_image,int hauteur_image,int low,int hight)
{
  int i;
  for(i=0;i<largeur_image*hauteur_image;i++)
    image[i]=(low<image[i] && image[i]<hight)?255:0;

}

void morpion_invert_image_grey(unsigned char*image,int largeur_image,int hauteur_image)
{
  int i;
  for(i=0;i<largeur_image*hauteur_image;i++)
    image[i]=255-image[i];

}


#define OP(x) \
  for(j=0;j<hauteur_image;j++)\
    for(i=0;i<largeur_image;i++){\
      image[j*largeur_image+i]=image[j*largeur_image+i] x frame[(taille_es_v+j)*(largeur_image+2*taille_es_h)+(taille_es_h+i)]?\
	image[j*largeur_image+i]:\
	frame[(taille_es_v+j)*(largeur_image+2*taille_es_h)+(taille_es_h+i)];\
    }
#define SUP OP(>)
#define INF OP(<)

void morpion_lines_off(unsigned char*image,int largeur_image,int hauteur_image)
{
  int i,j;
  int taille_es_v=2;
  int taille_es_h=2;
  unsigned char *frame;
  unsigned char image_orig[hauteur_image*largeur_image];
  int taille_cadre;

  memcpy(image_orig,image,sizeof(image_orig));

  taille_cadre=(largeur_image+2*taille_es_h)*(hauteur_image+2*taille_es_v);

  frame=(unsigned char*)malloc(taille_cadre);

  /* fermeture */

  //DILATION
  //decalage en haut
  memset(frame,0,taille_cadre);
  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[j*(largeur_image+2*taille_es_h)+taille_es_h],&image[j*largeur_image],largeur_image);
  SUP;
    
  //decalage en bas
  memset(frame,0,taille_cadre);
  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[(2*taille_es_v+j)*(largeur_image+2*taille_es_h)+taille_es_h],&image[j*largeur_image],largeur_image);
  SUP;

  //decalage a gauche
  memset(frame,0,taille_cadre);
  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[j*(largeur_image+2*taille_es_h)],&image[j*largeur_image],largeur_image);
  SUP;

  //decalage a droite
  memset(frame,0,taille_cadre);
  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[j*(largeur_image+2*taille_es_h)+(2*taille_es_h)],&image[j*largeur_image],largeur_image);
  SUP;

  
  // EROSION
  //decalage en haut
  memset(frame,255,taille_cadre);

  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[j*(largeur_image+2*taille_es_h)+taille_es_h],&image[j*largeur_image],largeur_image);
  INF;
    
  //decalage en bas
  memset(frame,255,taille_cadre);
  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[(2*taille_es_v+j)*(largeur_image+2*taille_es_h)+taille_es_h],&image[j*largeur_image],largeur_image);
  INF;

  //decalage a gauche
  memset(frame,255,taille_cadre);
  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[j*(largeur_image+2*taille_es_h)],&image[j*largeur_image],largeur_image);
  INF;

  //decalage a droite
  memset(frame,255,taille_cadre);
  for(j=0;j<hauteur_image;j++)
    memcpy(&frame[j*(largeur_image+2*taille_es_h)+(2*taille_es_h)],&image[j*largeur_image],largeur_image);
  INF;
  

}

/****************************************************
 * Procdure utilitaire permettant de visualiser 
 * la configurtation du morpion sur la sortie standard
 * ou d'imprimer une image dans un fichier pnm.
 */
void morpion_display_etat(unsigned char*etat)
{
  printf(" ------- \n");
  printf("| ");
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  printf("|\n");
  printf("| ");
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  printf("|\n");
  printf("| ");
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  *etat>0?printf("%d ",*etat):printf("  "); etat++;
  printf("|\n");
  printf(" ------- \n");

  return;
}

void morpion_save_image_grey(char*filename,unsigned char*image,int largeur_image,int hauteur_image)
{
  FILE*fp;
  unsigned char out[largeur_image*hauteur_image*3];
  int i;

  for(i=0;i<sizeof(out);i+=3){
    out[i]=image[i/3];
    out[i+1]=out[i];
    out[i+2]=out[i];
  }

  fp=fopen(filename,"w+");
  fprintf(fp, "P6\n%d %d\n255\n", largeur_image,hauteur_image);
  fwrite (out,sizeof(out), 1, fp);
  fclose(fp);

  return;
}
